@@ -6,17 +6,15 @@ from lxml import etree
 import json
 import pathlib
 
+DEFAULT_TRANSPORT = 'Symphony'
 
 class AriaService:
-    def __init__(self, base_url, cert_file, session_id, **kwargs):
+    def __init__(self, base_url, session_id, **kwargs):
         self._base_url = base_url
-        self._certificate = AriaService.read_cert(cert_file)
         self._token = None
         self._session_id = session_id
         self._timeout_default = 30 if 'timeout' not in kwargs else int(kwargs['timeout'])
-        app_cert_file = None if 'app_credentials' not in kwargs  else kwargs['app_credentials']
-        self._app_credentials = None if app_cert_file is None else AriaService.read_cert(app_cert_file)
-        self._headers = {}
+        self._headers = {'content-type':'application/json'}
 
     def _process_aria_message(self, res, attribute=None):
         try:
@@ -28,28 +26,35 @@ class AriaService:
                     return data['message']
                 else:
                     raise AriaError('No {} returned from server'.format(attribute), res)
-            elif attribute in data:
-                return data[attribute]
+            if not data.get(attribute, None) and not data.get('detail', None):
+                raise AriaError('Unrecognised server response', res)
+            elif data['detail'].get(attribute, None):
+                return data['detail'][attribute]
             else:
                 raise AriaError('Unrecognised server response', res)
         except ValueError as ve:
             raise AriaError(ve.args[0], res)
 
-    def authenticate(self):
+    def authenticate_app(self, key):
+        token = self._authenticate('appKey', key)
+        self._headers['Authorization'] = 'Bearer {}'.format(token)
+
+    def authenticate(self, key):
+        token = self._authenticate('userKey', key)
+        self._headers['Authorization'] = 'Bearer {}'.format(token)
+
+    def _authenticate(self, keyType, key):
         data = {
             "session": self._session_id,
-            "credentials": self._certificate.decode('utf-8')
+            "credentials": [{'transport': 'Symphony', keyType: key.decode('utf-8')}]
         }
-        if self._app_credentials is not None:
-            data['appCredentials'] = self._app_credentials.decode('utf-8')
         res = requests.post(
             self._base_url+'/auth',
             json=data,
             headers={'content-type': 'application/json'}
         )
         res.raise_for_status()
-        token = self._process_aria_message(res, 'token')
-        self._headers['Authorization'] = 'Bearer {}'.format(token)
+        return self._process_aria_message(res, 'token')
 
     def get_messages(self):
         res = requests.get(
@@ -73,29 +78,25 @@ class AriaService:
         res.raise_for_status()
         return self._process_aria_message(res)
 
-    def send_message(self, channel, message, **kwargs):
-        payload = {"channel": channel,
-                       "options": {
-                           'disableEncoding': None if 'disableEncoding' not in kwargs else kwargs['disableEncoding'],
-                           'passthrough': None if 'passthrough' not in kwargs else kwargs['passthrough']
-                       }
-                   }
+    def send_message(self, channel, raw_message, **kwargs):
+        message = {"channel": channel, "payload": {}}
         data = None if 'data' not in kwargs else kwargs['data']
         if isinstance(data, str):
             data = json.loads(data)
-        payload = AriaService.set_text(payload, message, data)
+
+        message = AriaService.set_text(message, raw_message, data)
         if 'attachment' in kwargs and kwargs['attachment'] is not None:
             if isinstance(kwargs['attachment'], tuple):
                 filename, fdata = kwargs['attachment']
-                payload['attachments'] = []
-                payload['attachments'].append(
+                message['attachments'] = []
+                message['attachments'].append(
                     {'name': filename,
                      'data': base64.b64encode(fdata).decode(),
                      'content-type': mimetypes.guess_type(filename)}
                 )
         if 'on_behalf' in kwargs and kwargs['on_behalf'] is not None:
-            payload['on_behalf'] = kwargs['on_behalf']
-        return requests.post('{}/message'.format(self._base_url), json=payload, headers=self._headers)
+            message['on_behalf'] = kwargs['on_behalf']
+        return requests.post('{}/message'.format(self._base_url), json=message, headers=self._headers)
 
     def delete_message(self, id):
         payload = {"id": id}
@@ -103,7 +104,6 @@ class AriaService:
         res.raise_for_status()
         return res
 
-
     def get_user(self, user_id, id_type='id', admin=False):
         params = {id_type: user_id}
         if admin:
@@ -117,9 +117,8 @@ class AriaService:
         return self._process_aria_message(res)
 
     def who_am_i(self):
-        params = {'session': True }
         res = requests.get(
-            self._base_url+'/user', params=params,
+            self._base_url+'/user/session',
             headers=self._headers,
             timeout=self._timeout_default
         )
@@ -159,13 +158,17 @@ class AriaService:
             headers=self._headers,
             timeout=self._timeout_default
         )
-        res.raise_for_status()
+        if res.status_code == requests.codes.not_found:
+            raise AriaError('RoomId {0} not found'.format(room_id), res)
+        else:
+            res.raise_for_status()
+
         return self._process_aria_message(res)
 
-    def create_room(self, room_name):
+    def create_room(self, room_name, room_description):
         res = requests.post(
             '{}/room'.format(self._base_url),
-            json={"name": room_name},
+            json={"name": room_name, "description": room_description},
             headers=self._headers,
             timeout=self._timeout_default
         )
@@ -184,19 +187,22 @@ class AriaService:
             params=params,
             timeout=self._timeout_default
         )
-        res.raise_for_status()
+        if res.status_code == requests.codes.not_found:
+            raise AriaError('Room {0} not found'.format(room_name), res)
+        else:
+            res.raise_for_status()
         return self._process_aria_message(res)
 
     '''Add a member or list of members to a room'''
     def add_room_members(self, room_name, member_email, **kwargs):
         users = []
         if not isinstance(member_email, list):
-            users.append({"email": member_email})
+            users.append(member_email)
         else:
             for user in member_email:
-                users.append({"email": user})
+                users.append(user)
 
-        data = {"users": users}
+        data = {"email": users}
         if 'on_behalf' in kwargs and kwargs['on_behalf'] is not None:
             data['on_behalf'] = kwargs['on_behalf']
 
@@ -206,19 +212,22 @@ class AriaService:
             headers=self._headers,
             timeout=self._timeout_default
         )
-        res.raise_for_status()
+        if res.status_code == requests.codes.not_found:
+            raise AriaError('Room {0} not found'.format(room_name), res)
+        else:
+            res.raise_for_status()
         return self._process_aria_message(res)
 
     '''Remove a member or list of members from a room'''
     def remove_room_members(self, room_name, member_email, **kwargs):
         users = []
         if not isinstance(member_email, list):
-            users.append({"email": member_email})
+            users.append(member_email)
         else:
             for user in member_email:
-                users.append({"email": user})
+                users.append(user)
 
-        data = {"users": users}
+        data = {"email": users}
         if 'on_behalf' in kwargs and kwargs['on_behalf'] is not None:
             data['on_behalf'] = kwargs['on_behalf']
 
@@ -228,7 +237,10 @@ class AriaService:
             headers=self._headers,
             timeout=self._timeout_default
         )
-        res.raise_for_status()
+        if res.status_code == requests.codes.not_found:
+            raise AriaError('Room {0} not found'.format(room_name), res)
+        else:
+            res.raise_for_status()
         return self._process_aria_message(res)
 
     def download_file(self, url, local_filename=None):
@@ -246,17 +258,14 @@ class AriaService:
     #    def send_stream_as_attachment(self, channel, attach, stream, disableEncoding=True, passthrough=True):
 
     @staticmethod
-    def set_text(msg, text, data=None):
+    def set_text(message, text, data=None):
         if isinstance(text, SymphonyMessageML):
-            msg['raw'] = {'message': text.__str__()}
-        elif text.startswith('<messageML>') or text.startswith('<div '):
-            msg['raw'] = {'message': text}
+            message['payload']['message'] = text.__str__()
         else:
-            msg['raw'] = {}
-            msg['text'] = text
+            message['payload']['message'] = text
         if data is not None:
-            msg['raw']['data'] = data
-        return msg
+            message['payload']['data'] = json.dumps(data)
+        return message
 
     @staticmethod
     def read_cert(filename):
